# inspired by example at
# https://github.com/picotech/picosdk-python-wrappers/blob/master/usbtc08Examples/tc08StreamingModeExample.py
#
# API docs
# https://www.picotech.com/download/manuals/usb-tc08-thermocouple-data-logger-programmers-guide.pdf

import signal
from threading import Timer
from tc08_device import TC08
from influxdb_bridge import InfluxDBBridge

# how often does Monitor check for new data from the TC08 driver
LOG_INTERVAL_SEC = 5

DEBUG = False


def dprint(arg):
    if DEBUG:
        print(arg)


class Monitor(Timer):
    def __init__(self, interval):
        # Timer constructer expects a callback function, but we don't care
        # about that, so just pass None
        super(Monitor, self).__init__(interval, None)
        self.idb = None
        self.tc08s = list()

    # override Timer's run method to loop on our injest_data function
    # wait will block while loop for timeout interval
    # calling timer.cancel() sets the finish flag and exits the loop
    def run(self):
        dprint("in timer.run method")
        while not self.finished.wait(timeout=self.interval):
            self.do_work()

    # connect as many tc08's as we can find
    def setup(self):
        searching = True
        while searching:
            device = TC08.get_any_tc08()
            if device is None:
                dprint(f"No more TC08s to dicsover")
                searching = False
            else:
                dprint(f"Found TC08: {device}")
                self.tc08s.append(device)

        # setup and run
        for device in self.tc08s:
            device.setup()
            device.run(LOG_INTERVAL_SEC * 1000)

        # open connection to influxdb
        self.idb = InfluxDBBridge()
        self.idb.connect()

        dprint("end setup")

    def cleaup(self):
        for device in self.tc08s:
            device.stop()
            device.close()

    def do_work(self):
        points = []
        for device in self.tc08s:
            serial = device.get_serial()
            data = device.get_streaming_data()
            for channel in range(0, 9):
                for reading in data[channel]:
                    (temperature, time_ms, overflow) = reading
                    point = self.make_point(
                        serial,
                        channel,
                        temperature,
                        time_ms,
                        overflow,
                    )
                    points.append(point)
        self.idb.commit(points)

    def make_point(self, serial, channel, temperature, time_ms, overflow):
        json_body = {
            "measurement": "temperature",
            "tags": {"channel": channel, "tc08-serial": serial},
            "time": time_ms,
            "fields": {"temperature": temperature, "overflow": overflow},
        }
        return json_body

    def start_monitor(self):
        self.setup()
        self.start()

    def cancelISR(self, *args, **kwargs):
        dprint("cancelISR called")
        self.cancel()
        for device in self.tc08s:
            device.stop()
            device.close()


if __name__ == "__main__":
    monitor = Monitor(LOG_INTERVAL_SEC)
    monitor.start_monitor()
    dprint("monitor running")
    signal.signal(signal.SIGINT, monitor.cancelISR)
    signal.signal(signal.SIGTERM, monitor.cancelISR)
    monitor.join()
